
#include "PlayerInputSystem.h"
#include "TransformableComponent.h"
#include "PlayerComponent.h"
#include "VelocityComponent.h"
#include "EntityFactory.h"
#include "BeamComponent.h"
#include "Constants.h"
#include "Utility.h"
#include "SoundManager.h"

const float PlayerInputSystem::playerMoveSpeed = 400.0f;
const float PlayerInputSystem::playerSlowMoveSpeed = 200.0f;
const float PlayerInputSystem::playerTurnSpeed = 180.0f;
const float PlayerInputSystem::playerSlowTurnSpeed = 120.0f;
const sf::Time PlayerInputSystem::powerDownTime = sf::seconds(1.5f);
const sf::Vector2f PlayerInputSystem::maxPlayerMoveSpeed = sf::Vector2f(120.0f, 800.0f);
const sf::Vector2f PlayerInputSystem::maxPlayerSlowMoveSpeed = sf::Vector2f(90.0f, 400.0f);

PlayerInputSystem::PlayerInputSystem()
{
	addComponentType<VelocityComponent>();
	addComponentType<TransformableComponent>();
	addComponentType<PlayerComponent>();

	keyDown.resize(sf::Keyboard::KeyCount);
	mouseDown.resize(sf::Mouse::ButtonCount);
};

void PlayerInputSystem::initialize() {
	velocityMapper.init(*world);
	TransformableMapper.init(*world);
	playerMapper.init(*world);
};

void PlayerInputSystem::processEntity(artemis::Entity &e) {
	PlayerComponent* playerMap = playerMapper.get(e);
	sf::Vector2f playerPos = TransformableMapper.get(e)->transformable->getPosition();
	sf::Vector2f movement;
	float moveSpeed = playerMoveSpeed;
	float turnSpeed = playerTurnSpeed;
	sf::Vector2f maxMoveSpeed = maxPlayerMoveSpeed;

	if(keyDown[playerMap->rightKey]) {
		movement.x += 1.0f;
	}
	if(keyDown[playerMap->leftKey]) {
		movement.x -= 1.0f;
	}

	if(keyDown[playerMap->downKey]) {
		movement.y -= 1.0f;
	}
	if(keyDown[playerMap->upKey]) {
		movement.y += 1.0f;
	}

	if(mouseDown[playerMap->shootBullet] && playerMap->bulletShotTimer <= sf::Time::Zero) {
		//If the mouse is down and we can shoot a bullet, do so
		EntityFactory::CreateBullet(*world, sf::Vector2f(0.0f, 12.0f));
		EntityFactory::CreateBullet(*world, sf::Vector2f(0.0f, 24.0f));
		EntityFactory::CreateBullet(*world, sf::Vector2f(0.0f, -12.0f));
		EntityFactory::CreateBullet(*world, sf::Vector2f(0.0f, -24.0f));
		playerMap->bulletShotTimer = sf::seconds(0.2f);
		printf("firing bullet\n");
	} else {
		playerMap->bulletShotTimer = std::max(sf::Time::Zero,
			playerMap->bulletShotTimer - sf::seconds(world->getDelta()));
	}

	if(mouseDown[playerMap->shootBeam] && playerMap->beamTimer <= sf::Time::Zero && !playerMap->shootingBeam) {
		//If the mouse is down and we can shoot a beam, then do so
		artemis::Entity& beam = EntityFactory::CreateBeam(*world, playerPos, mousePt);
		playerMap->beamTimer = powerDownTime;
		playerMap->shootingBeam = true;
	}

	if(!mouseDown[playerMap->shootBeam] && playerMap->shootingBeam) {
		//If the mouse is not down and the beam was shooting, stop it
		artemis::ImmutableBag<artemis::Entity*> * beamBag = 
			world->getGroupManager()->getEntities(constants::beamGroup);

		for(int i = 0; i < beamBag->getCount(); i++) {
			BeamComponent* beam = beamBag->get(i)->getComponent<BeamComponent>();
			beam->curSound->stop();
			world->deleteEntity(*beamBag->get(i));
		}

		SoundManager::PlaySound("assets/sound/powerdown.ogg",false);
		playerMap->shootingBeam = false;
	}

	if(mouseDown[sf::Mouse::Middle]) {
		EntityFactory::CreateStaticTest(*world, sf::Vector2f(getRandom(0, 800), getRandom(0,600)));
	}

	if(!playerMap->shootingBeam && playerMap->beamTimer > sf::Time::Zero) {
		//If the beam is not shooting and the timer's still running, keep counting down
		playerMap->beamTimer -= sf::seconds(world->getDelta());
	} else if(playerMap->shootingBeam && magnitude(movement) != 0) {
		//If the beam is still shooting and we're moving, move slower
		moveSpeed = playerSlowMoveSpeed;
		turnSpeed = playerSlowTurnSpeed;
		maxMoveSpeed = maxPlayerSlowMoveSpeed;
	}

	movement.x = sign(movement.x)*turnSpeed;
	movement.y = sign(movement.y)*moveSpeed;

	velocityMapper.get(e)->accel = movement;
	velocityMapper.get(e)->velocityMax = maxMoveSpeed;

	/*sf::Vector2f pos = positionMapper.get(e)->pos;
	float angle = atan2(mousePt.y - pos.y, mousePt.x - pos.x);
	velocityMapper.get(e)->velocity = sf::Vector2f(
		playerMoveSpeed*cos(angle), playerMoveSpeed*sin(angle));*/
	//positionMapper.get(e)->pos = sf::Vector2f(mousePt.x, mousePt.y);
};

void PlayerInputSystem::SetMouse(sf::Vector2f mouse) {
	mousePt = mouse;
}

void PlayerInputSystem::KeyDown(sf::Keyboard::Key key) {
	if(key != sf::Keyboard::Unknown) {
		keyDown[key] = true;
	} else {
		fprintf(stderr, "WARNING: Received unknown key press\n");
	}
}

void PlayerInputSystem::KeyRelease(sf::Keyboard::Key key) {
	if(key != sf::Keyboard::Unknown) {
		keyDown[key] = false;
	} else {
		fprintf(stderr, "WARNING: Received unknown key release\n");
	}
}

void PlayerInputSystem::MousePress(sf::Mouse::Button button) {
	mouseDown[button] = true;
}

void PlayerInputSystem::MouseRelease(sf::Mouse::Button button) {
	mouseDown[button] = false;
}
